# -*- coding: utf-8 -*-

###########################################
# maintainer: <plnick@vuplus-support.org> #
##    http://www.vuplus-support.org      ##
###########################################

#This plugin is free software, you are allowed to
#modify it (if you keep the license),
#but you are not allowed to distribute/publish
#it without source code (this version and your modifications).
#This means you also have to distribute
#source code of your modifications.

# camera icon is taken from oxygen icon theme for KDE 4

# modified by Smokey 25-5-2015
# in this version it's possible to select an other button to make a screen shot.
# a Dutch translation file is added, and some translations have changed in the other translation files.

from enigma import eActionMap, eConsoleAppContainer
from Components.ActionMap import ActionMap
from Components.ConfigList import ConfigListScreen
from Components.config import config, getConfigListEntry, ConfigSubsection, ConfigSelection, ConfigEnableDisable, ConfigYesNo, ConfigInteger
from Components.Console import Console
from Components.Label import Label
from Plugins.Plugin import PluginDescriptor
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Tools.Notifications import AddNotification
from Tools.BoundFunction import boundFunction
from datetime import datetime
from time import time as systime
from os import system, path, mkdir, makedirs
from __init__ import _

pluginversion = "Version: 0.2-mod-r2"
config.plugins.shootyourscreen = ConfigSubsection()
config.plugins.shootyourscreen.enable = ConfigEnableDisable(default=True)
config.plugins.shootyourscreen.switchhelp = ConfigYesNo(default=True)
config.plugins.shootyourscreen.path = ConfigSelection(default = "/media/hdd", choices = [("/media/hdd"), ("/media/usb"), ("/media/hdd1"), ("/media/usb1"), ("/tmp", "/tmp")])
config.plugins.shootyourscreen.pictureformat = ConfigSelection(default = "bmp", choices = [("bmp", "bmp"), ("-j", "jpg"), ("-p", "png")])
config.plugins.shootyourscreen.jpegquality = ConfigSelection(default = "100", choices = [("10"), ("20"), ("40"), ("60"), ("80"), ("100")])
config.plugins.shootyourscreen.picturetype = ConfigSelection(default = "all", choices = [("all", "OSD + Video"), ("-v", "Video"), ("-o", "OSD")])
config.plugins.shootyourscreen.picturesize = ConfigSelection(default = "default", choices = [("default", _("Skin resolution")), ("-r 480", "480"), ("-r 576", "576"), ("-r 720", "720"), ("-r 1280", "1280"), ("-r 1920", "1920")])
config.plugins.shootyourscreen.timeout = ConfigSelection(default = "3", choices = [("1", "1 sec"), ("3", "3 sec"), ("5", "5 sec"), ("10", "10 sec"), ("off", _("no message")), ("0", _("no timeout"))])
config.plugins.shootyourscreen.buttonchoice = ConfigSelection(default = "138", choices = [("113", _("Mute")), ("138", _("Help")), ("362", _("Timer")), ("377", _("TV")), ("385", _("Radio")), ("388", _("Text")), ("392", _("Audio")), ("398", _("Red")), ("399", _("Green")), ("400", _("Yellow")), ("401", _("Blue"))])
config.plugins.shootyourscreen.dummy = ConfigSelection(default = "1", choices = [("1", " ")])

def getPicturePath():
	picturepath = config.plugins.shootyourscreen.path.value
	if picturepath.endswith('/'):
		picturepath = picturepath + 'screenshots'
	else:
		picturepath = picturepath + '/screenshots'

	try:
		if (path.exists(picturepath) == False):
			makedirs(picturepath)
	except OSError:
		self.session.open(MessageBox, _("Sorry, your device for screenshots is not writeable.\n\nPlease choose another one."), MessageBox.TYPE_INFO, timeout = 10)
	return picturepath

class getScreenshot:
	def __init__(self):
		self.ScreenshotConsole = Console()
		self.previousflag = 0
		eActionMap.getInstance().bindAction('', -0x7FFFFFFF, self.screenshotKey)

	def screenshotKey(self, key, flag):
		selectedbutton = int(config.plugins.shootyourscreen.buttonchoice.value)
		if config.plugins.shootyourscreen.enable.value:
			if key == selectedbutton:
				if not config.plugins.shootyourscreen.switchhelp.value:
					if flag == 3:
						self.previousflag = flag
						self.grabScreenshot()
						return 1
					if self.previousflag == 3 and flag ==1:
						self.previousflag = 0
						return 1
				else:
					if flag == 0:
						return 1
					if flag == 3:
						self.previousflag = flag
						return 0
					if flag == 1 and self.previousflag == 0:
						self.grabScreenshot()
						return 1
					if self.previousflag == 3 and flag ==1:
						self.previousflag = 0
						return 0
		return 0

	def grabScreenshot(self, ret = None):
		filename = self.getFilename()
		print "[ShootYourScreen] grab screenshot to %s" % filename
		cmd = "grab"
		if not config.plugins.shootyourscreen.picturetype.value == "all":
			cmdoptiontype = " " + str(config.plugins.shootyourscreen.picturetype.value)
			cmd += cmdoptiontype
		if not config.plugins.shootyourscreen.picturesize.value == "default":
			cmdoptionsize = " " + str(config.plugins.shootyourscreen.picturesize.value)
			cmd += cmdoptionsize
		if not config.plugins.shootyourscreen.pictureformat.value == "bmp":
			cmdoptionformat = " " + str(config.plugins.shootyourscreen.pictureformat.value)
			cmd += cmdoptionformat
			if config.plugins.shootyourscreen.pictureformat.value == "-j":
				cmdoptionquality = " " + str(config.plugins.shootyourscreen.jpegquality.value)
				cmd += cmdoptionquality
		cmd += " %s" % filename
		extra_args = (filename)
		self.ScreenshotConsole.ePopen(cmd, self.gotScreenshot, extra_args)

	def gotScreenshot(self, result, retval, extra_args = None):
		if extra_args is not None:
			filename = extra_args
		else:
			filename = ""
		
		if not config.plugins.shootyourscreen.timeout.value == "off":
			messagetimeout = int(config.plugins.shootyourscreen.timeout.value)
			if retval == 0:
				msg_text = _("Screenshot successfully saved as:\n%s") % filename
				msg_type = MessageBox.TYPE_INFO
			else:
				msg_text = _("Grabbing Screenshot failed !!!")
				msg_type = MessageBox.TYPE_ERROR
			AddNotification(MessageBox, msg_text, MessageBox.TYPE_INFO, timeout = messagetimeout)
		else:
			pass

	def getFilename(self):
		now = systime()
		now = datetime.fromtimestamp(now)
		now = now.strftime("%Y-%m-%d_%H-%M-%S")

		screenshottime = "screenshot_" + now
		
		if config.plugins.shootyourscreen.pictureformat.value == "bmp":
			fileextension = ".bmp"
		elif config.plugins.shootyourscreen.pictureformat.value == "-j":
			fileextension = ".jpg"
		elif config.plugins.shootyourscreen.pictureformat.value == "-p":
			fileextension = ".png"

		picturepath = getPicturePath()
		if picturepath.endswith('/'):
			screenshotfile = picturepath + screenshottime + fileextension
		else:
			screenshotfile = picturepath + '/' + screenshottime + fileextension
		return screenshotfile

class ShootYourScreenConfig(Screen, ConfigListScreen):
	skin = """
		<screen position="center,center" size="650,400" title="ShootYourScreen for VU+" >
		<widget name="config" position="10,10" size="630,350" scrollbarMode="showOnDemand" />
		<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/ShootYourScreen/pic/button_red.png" zPosition="2" position="10,370" size="25,25" alphatest="on" />
		<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/ShootYourScreen/pic/button_green.png" zPosition="2" position="130,370" size="25,25" alphatest="on" />
		<ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/ShootYourScreen/pic/button_yellow.png" zPosition="2" position="270,370" size="25,25" alphatest="on" />
		<widget name="buttonred" position="40,372" size="100,20" valign="center" halign="left" zPosition="2" foregroundColor="white" font="Regular;18"/>
		<widget name="buttongreen" position="160,372" size="100,20" valign="center" halign="left" zPosition="2" foregroundColor="white" font="Regular;18"/>
		<widget name="buttonyellow" position="300,372" size="200,20" valign="center" halign="left" zPosition="2" foregroundColor="white" font="Regular;18"/>
		</screen>"""

	def __init__(self,session):
		self.session = session
		Screen.__init__(self,session)
		
		self.createConfigList()

		ConfigListScreen.__init__(self, self.list, session = self.session, on_change = self.changedEntry)

		self["buttonred"] = Label(_("Exit"))
		self["buttongreen"] = Label(_("Save"))
		self["buttonyellow"] = Label(_("Default"))
		self["setupActions"] = ActionMap(["SetupActions", "ColorActions"],
			{
				"green": self.keyGreen,
				"red": self.cancel,
				"yellow": self.revert,
				"cancel": self.cancel,
				"ok": self.keyGreen,
			}, -2)
			
		self.onShown.append(self.setWindowTitle)

	def setWindowTitle(self):
		self.setTitle(_("ShootYourScreen for VU+ STB - %s") % pluginversion)

	def createConfigList(self):
		self.list = []
		self.list.append(getConfigListEntry(_("Enable ShootYourScreen :"), config.plugins.shootyourscreen.enable))
		if config.plugins.shootyourscreen.enable.value == True:
			self.list.append(getConfigListEntry(_("Screenshot of :"), config.plugins.shootyourscreen.picturetype))
			self.list.append(getConfigListEntry(_("Format for screenshots :"), config.plugins.shootyourscreen.pictureformat))
			if config.plugins.shootyourscreen.pictureformat.value == "-j":
				self.list.append(getConfigListEntry(_("Quality of jpg picture :"), config.plugins.shootyourscreen.jpegquality))
			self.list.append(getConfigListEntry(_("Picture size (width) :"), config.plugins.shootyourscreen.picturesize))
			self.list.append(getConfigListEntry(_("Path for screenshots :"), config.plugins.shootyourscreen.path))
			self.list.append(getConfigListEntry(_("Select a button to take a screenshot :"), config.plugins.shootyourscreen.buttonchoice))
			check = config.plugins.shootyourscreen.buttonchoice.value
			if check == "113":
				buttonname = (_("Mute"))
			elif check == "138":
				buttonname = (_("Help"))
			elif check == "362":
				buttonname = (_("Timer"))
			elif check == "377":
				buttonname = (_("TV"))
			elif check == "385":
				buttonname = (_("Radio"))
			elif check == "388":
				buttonname = (_("Text"))
			elif check == "392":
				buttonname = (_("Audio"))
			elif check == "398":
				buttonname = (_("Red"))
			elif check == "399":
				buttonname = (_("Green"))
			elif check == "400":
				buttonname = (_("Yellow"))
			elif check == "401":
				buttonname = (_("Blue"))
			if check == '398' or check == '399' or check == '400':
				self.list.append(getConfigListEntry(_("Only button ' ") + buttonname + _(" long ' can be used."), config.plugins.shootyourscreen.dummy))
				config.plugins.shootyourscreen.switchhelp.setValue(0)
			else:
				self.list.append(getConfigListEntry(_("Use the ' ") + buttonname + _(" ' button instead of ' ") + buttonname + _(" long ' :"), config.plugins.shootyourscreen.switchhelp))					
			self.list.append(getConfigListEntry(_("Timeout for info message :"), config.plugins.shootyourscreen.timeout))

	def changedEntry(self):
		self.createConfigList()
		self["config"].setList(self.list)

	def save(self):
		for x in self["config"].list:
			x[1].save()
		self.changedEntry()

	def keyGreen(self):
		self.save()
		self.close(False,self.session)

	def cancel(self):
		if self["config"].isChanged():
			self.session.openWithCallback(self.cancelConfirm, MessageBox, _("Really close without saving settings?"), MessageBox.TYPE_YESNO, default = True)
		else:
			for x in self["config"].list:
				x[1].cancel()
			self.close(False,self.session)

	def cancelConfirm(self, result):
		if result is None or result is False:
			print "[ShootYourScreen] Cancel not confirmed."
		else:
			print "[ShootYourScreen] Cancel confirmed. Configchanges will be lost."
			for x in self["config"].list:
				x[1].cancel()
			self.close(False,self.session)

	def revert(self):
		self.session.openWithCallback(self.keyYellowConfirm, MessageBox, _("Reset ShootYourScreen settings to defaults?"), MessageBox.TYPE_YESNO, timeout = 20, default = True)

	def keyYellowConfirm(self, confirmed):
		if not confirmed:
			print "[ShootYourScreen] Reset to defaults not confirmed."
		else:
			print "[ShootYourScreen] Setting Configuration to defaults."
			config.plugins.shootyourscreen.enable.setValue(1)
			config.plugins.shootyourscreen.switchhelp.setValue(1)
			config.plugins.shootyourscreen.path.setValue("/media/hdd")
			config.plugins.shootyourscreen.pictureformat.setValue("bmp")
			config.plugins.shootyourscreen.jpegquality.setValue("100")
			config.plugins.shootyourscreen.picturetype.setValue("all")
			config.plugins.shootyourscreen.picturesize.setValue("default")
			config.plugins.shootyourscreen.timeout.setValue("3")
			config.plugins.shootyourscreen.buttonchoice.setValue("138")
			self.save()

def autostart(reason, **kwargs):
	if kwargs.has_key("session") and reason == 0:
		print "[ShootYourScreen] start...."
		getScreenshot()

def startSetup(session, **kwargs):
		print "[ShootYourScreen] start configuration"
		session.open(ShootYourScreenConfig)

def Plugins(**kwargs):
			return [PluginDescriptor(where = PluginDescriptor.WHERE_SESSIONSTART, fnc = autostart),
				PluginDescriptor(name = "ShootYourScreen Setup", description = _("make Screenshots with your VU+"), where = [PluginDescriptor.WHERE_PLUGINMENU, PluginDescriptor.WHERE_EXTENSIONSMENU], icon="shootyourscreen.png", fnc=startSetup)]
